function [ lpt ] = logp_argmax( Xt, Xd, yd, ...
    rho, kernel, K_prior, y_prior, xi )
%LOGP_ARGMAX Computes the unnormalized log-posterior density of the argmax
%   at given points.
%
%   LOGP_ARGMAX( Xt, Xd, yd ) computes the un-normalized log-posterior
%   probability density of the maximizing argument at test locations Xt
%   given previously observed values yd at locations Xd. Default kernels
%   and precisions are used. 
%
%   Xt and Xd are matrices of row vectors of sizes mt x n and md x n
%   respectively, where n is the dimension of the input space. 'yd' is
%   a md x 1 column vector containing the observed function values at
%   locations specified in Xd.
%
%   LOGP_ARGMAX( Xt, Xd, yd, rho, kernel, K_prior, y_prior, xi)
%   does the same, but using precision scale 'rho' (scalar),
%   kernel function 'kernel' (function handle), prior precision function
%   'K_prior' (function handle) and prior function estimate 'y_prior'
%   (function handle) and prior number of samples 'xi'.
%
%   Code (c) 2012 Pedro A. Ortega, Tim Genewein, Jordi Gray-Moya.
%
%   Reference: 
%   Ortega, P.A., Grau-Moya, J., Genewein, T., Balduzzi, D. and Braun, D.A.
%   "A Nonparametric Conjugate Prior Distribution for the Maximizing
%   Argument of a Noisy Function."
%   Neural Information Processing Systems (NIPS) 2012

%% Check arguments.

% If hyperparameters are not given, then use default values.
if nargin <= 3;
    rho = 1;
    xi  = 1;
    kernel = @(x1, x2) exp(-sum((x1-x2).^2)/0.001);
    K_prior = @(x) 1;
    y_prior = @(x) exp(-sum(x.^2));
end;

% Determine sizes.
[Xd_m, Xd_n] = size(Xd);
[Xt_m, Xt_n] = size(Xt);

% Check whether these sizes make any sense.
if (Xd_n ~= 0 && Xd_n ~= Xt_n)
    error('Xd and Xt must have the same number of columns.');
end;
if (Xd_m ~= length(yd))
    error('Xd and yd must have the same number of rows.');
end;

%% Construct pseudo-observations.

K0 = zeros(Xt_m, 1); % Prior intensity vector.
y0 = zeros(Xt_m, 1); % Prior precision vector.
for i=1:Xt_m;
    K0(i) = K_prior(Xt(i,:));
    y0(i) = y_prior(Xt(i,:));
end;

%% Construct sufficient statistics.

if (Xd_n == 0)
    % No data points - use zero kernel matrix.
    neff = xi;
    fest = y0;
else
    % Compute kernel matrix for test points.
    Kt = zeros(Xt_m, Xd_m);
    for i=1:Xt_m;
        for j=1:Xd_m;
            Kt(i, j) = kernel(Xt(i,:), Xd(j,:));
        end;
    end;
    
    % Compute kernel matrix for data points.
    Kd = zeros(Xd_m, Xd_m);
    for i=1:Xd_m;
        for j=1:Xd_m;
            Kd(i, j) = kernel(Xd(i,:), Xd(j,:));
        end;
    end;
    
    neff = xi + Xd_m * sum(diag(Kd)) / sum(sum(Kd));
    fest = (Kt * yd + K0 .* y0) ./ (sum(Kt, 2) + K0);
end;

%% Calculate estimate.

lpt = rho * neff * fest;

end

