function [hFig] = test_argmax(Nd)
%TEST_ARGMAX Tests the argmax prior on a 1-dimensional objective function.
%
%   TEST_ARGMAX()
%   TEST_ARGMAX(Nd)
%   Produces a plot containing a 1-dimensional noisy objective function
%   plus a plot of the posterior distribution over the maximum for Nd 
%   uniformly drawn test locations.
%
%   If Nd is not specified, then default values are chosen.
%
%   Code (c) 2012 Pedro A. Ortega, Tim Genewein, Jordi Gray-Moya.
%
%   Reference: 
%   Ortega, P.A., Grau-Moya, J., Genewein, T., Balduzzi, D. and Braun, D.A.
%   "A Nonparametric Conjugate Prior Distribution for the Maximizing
%   Argument of a Noisy Function."
%   Neural Information Processing Systems (NIPS) 2012

%% Objective function.

f = @(x) cos(2*x+3/2*pi) + sin(6*x+3/2*pi);

%% Define hyper parameters.

rho     = 0.1;
xi      = 10;
kernel  = @(x1, x2) exp(-sum((x1-x2).^2)/0.3^2);
K_prior = @(x) 5;
y_prior = @(x) -0.1 * sum((x-1.5)^2/0.5^2) + 1;

%% Test points.

xdiv = 0.01;
Xt = [0:xdiv:3]';

%% Data points.

if (nargin < 1)
    Nd = 30;
end;

if (Nd > 0)
    Xd = 3 * rand(Nd, 1); 
    yd = f(Xd(:,1)) + (5*sin(Xd(:,1)).^2+1).*randn(Nd, 1);
else
    Xd = [];
    yd = [];
end

%% Compute posterior.

lt = logp_argmax(Xt, Xd, yd, rho, kernel, K_prior, y_prior, xi);
pt = exp(lt-max(lt)) / sum( exp(lt-max(lt)) );

%% Compute maximum.
[maxval, maxidx] = max(f(Xt));
maxarg = Xt(maxidx,:);

%% Plot.

hFig = figure;
set(hFig, 'Position', [300 300 1200 500]);
grey = 0.8 * ones(1, 3);

% Noisy function.

subplot(1, 2, 1);
xx = [Xt; flipud(Xt)];
yy = [f(Xt(:,1)) + (5*sin(Xt(:,1)).^2+1); flipud(f(Xt(:,1)) - (5*sin(Xt(:,1)).^2+1))];
fill(xx, yy, grey, 'EdgeColor', 'none');
hold on;
plot(Xt, f(Xt), 'k');
plot(Xd, yd, 'ro');
plot([maxarg, maxarg], [-20, 20], 'r--');
axis([0, 3, -20, 20]);
set(gca, 'YTick', (-20:10:20));
title('Noisy Function and Samples')

% Posterior.

subplot(1, 2, 2);
fill([0; Xt; 3], [0; pt/xdiv; 0], grey);
hold on;
plot([maxarg, maxarg], [0, 10], 'r--');
axis([0, 3, 0, 5]);
set(gca, 'YTick', (0:1:5));
title(sprintf('Posterior for %d Samples', Nd));

end