function [hFig] = test_argmax_3d(Nd)
%TEST_ARGMAX_3D Tests the argmax prior on a 2 dimensional objective
%    function.
%
%   TEST_ARGMAX_3D()
%   TEST_ARGMAX_3D(Nd)
%   Produces a plot containing a 2-dimensional noisy objective function
%   (peaks) plus a plot of the posterior distribution over the maximum
%   for Nd uniformly drawn test locations.
%
%   If Nd is not specified, then default values are chosen.
%
%   Code (c) 2012 Pedro A. Ortega, Tim Genewein, Jordi Gray-Moya.
%
%   Reference: 
%   Ortega, P.A., Grau-Moya, J., Genewein, T., Balduzzi, D. and Braun, D.A.
%   "A Nonparametric Conjugate Prior Distribution for the Maximizing
%   Argument of a Noisy Function."
%   Neural Information Processing Systems (NIPS) 2012


%% Objective function (peaks).

pf = @(x, y) 3*(1-x).^2.*exp(-(x.^2) - (y+1).^2) ... 
   - 10*(x/5 - x.^3 - y.^5).*exp(-x.^2-y.^2) ... 
   - 1/3*exp(-(x+1).^2 - y.^2);

%% Hyperparameters.

rho     = 0.3;
xi      = 1;
kernel  = @(x1, x2) exp(-sum((x1-x2).^2)/0.5^2);
K_prior = @(x) 10;
y_prior = @(x) -0.1 * sum(x.^2) + 1;

%% Test points.

Ng = 50;
Xt = zeros(Ng^2, 2);
for i=1:Ng;
    for j=1:Ng;
        k = (i-1)*Ng + j;
        Xt(k,:) = 6*[i,j]/Ng - 3;
    end;
end;

%% Data points.

if (nargin < 1)
    Nd = 30;
end;

if (Nd > 0)
    Xd = 6 * rand(Nd, 2) - 3; 
    yd = pf(Xd(:,1), Xd(:,2)) + randn(Nd, 1);
else
    Xd = [];
    yd = [];
end

%% Compute posterior.

lt = logp_argmax(Xt, Xd, yd, rho, kernel, K_prior, y_prior, xi);
pt = exp(lt-max(lt)) / sum( exp(lt-max(lt)) );

%% Plot.

% Domain grid and z-values of posterior and function.
[xx, yy] = meshgrid(6*(1:Ng)/Ng-3, 6*(1:Ng)/Ng-3);
zz = zeros(Ng, Ng);
ff = zeros(Ng, Ng);
for i=1:Ng;
    for j=1:Ng;
        zz(j, i) = pt((i-1)*Ng + j);
        ff(i, j) = pf(xx(i, j), yy(i,j));
    end;
end;

% Plot function and posterior.
hFig = figure;
set(hFig, 'Position', [300 300 1200 500]);

subplot(1, 2, 1);
surf(xx, yy, ff);
title('Objective Function (peaks)');

subplot(1, 2, 2);
surf(xx, yy, zz);
title(sprintf('Posterior for %d Samples', Nd));

end